import os
from jinja2 import Template
from playwright.async_api import async_playwright

async def pic_generator(data):
    data = {
        'roleName'                    : data['roleName'],
        'level'                       : data['level'],
        'server'                      : data['server'],
        'roleId'                      : data['roleId'],
        'roleScore'                   : data['roleScore'],
        'roleNum'                     : data['roleNum'],
        'fashionCollectionPercent'    : data['fashionCollectionPercent'],
        'actionValue'                 : data['actionValue'],
        'actionRefreshTimeStamp'      : data['actionRefreshTimeStamp'],
        'blackCardNum'                : data['blackCardNum'],
        'developResourceNum'          : data['developResourceNum'],
        'bossRefreshTimeStamp'        : data['bossRefreshTimeStamp'],
        'bossBlackCard'               : data['bossBlackCard'],
        'transfiniteNum'              : data['transfiniteNum'],
        'transfiniteRefreshTimeStamp' : data['transfiniteRefreshTimeStamp'],
        'arenaRefreshTimeStamp'       : data['arenaRefreshTimeStamp'],
        'arenaBlackCard'              : data['arenaBlackCard'],
        'strongHoldRate'              : data['strongHoldRate'],
        'strongHoldTimeStamp'         : data['strongHoldTimeStamp']
    }


    parent_parent_dir = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
    static_dir = os.path.join(parent_parent_dir, 'Static')
    rendered_template_path = os.path.join(static_dir, 'Outputs', 'rendered_template.html')
    pns_result_path = os.path.join(static_dir, 'pnsResult.html')
    
    with open(pns_result_path, 'r', encoding='utf-8') as file:
        template_str = file.read()
    template = Template(template_str)
    rendered_html = template.render(data)
    with open(rendered_template_path, 'w', encoding='utf-8') as file:
        file.write(rendered_html)

    async with async_playwright() as p:
        browser = await p.chromium.launch(headless=True, args=["--disable-gpu"])
        context = await browser.new_context()
        page = await context.new_page()


        await page.set_viewport_size({"width": 1080, "height": 1920})
        await page.goto(rendered_template_path)
        await page.wait_for_load_state("networkidle")
        screenshot_binary = await page.screenshot(type="png")
        await page.close()
        await browser.close()
    
    return screenshot_binary



import json
import time
from .pns_detail_handler import getPunishingResource, get_monthly_resource, get_pns_game_account, get_punishing_account_info
from ..pns_dao import UserInfoManagement
from ..calculate_time_stamp import calculate_time_stamp
from .pns_pic_render import pic_generator

async def pns_data_handler(data_row):
    pns_result = {}
    user_token        = data_row[4] #获取token
    token_data        = json.loads(user_token)['data']['token']
    pns_detail        = getPunishingResource(token_data)
    pns_resource      = get_monthly_resource(pns_detail['data']['roleId'], token_data)
    pns_role_list     = get_punishing_account_info(token_data)
    pns_account       = get_pns_game_account(pns_detail['data']['roleId'], pns_detail['data']['serverId'], token_data)
    current_timestamp = int(time.time())


    pns_result['roleName']                    = pns_detail['data']['roleName']
    pns_result['level']                       = (str(pns_account['data']['pointAfter']) + "勋阶") if pns_account['data']['roleLevel'] == 120 else (str(pns_account['data']['roleLevel']) + "级")
    pns_result['server']                      = pns_account['data']['serverName'] + "服"
    pns_result['roleId']                      = pns_detail['data']['roleId']
    pns_result['roleScore']                   = pns_role_list['data'][0]['roleScore']
    pns_result['roleNum']                     = pns_role_list['data'][0]['roleNum']
    pns_result['fashionCollectionPercent']    = str(int(pns_role_list['data'][0]['fashionCollectionPercent'] * 100)) + '%'
    pns_result['actionValue']                 = pns_detail['data']['actionData']['value']
    pns_result['actionRefreshTimeStamp']      = calculate_time_stamp(pns_detail['data']['actionData']['refreshTimeStamp'], current_timestamp)
    pns_result['blackCardNum']                = pns_resource['data']['currentBlackCard']
    pns_result['developResourceNum']          = pns_resource['data']['currentDevelopResource']
    pns_result['bossRefreshTimeStamp']        = calculate_time_stamp(pns_detail['data']['bossData'][0]['refreshTimeStamp'],current_timestamp)
    pns_result['bossBlackCard']               = pns_detail['data']['bossData'][0]['value']
    pns_result['transfiniteNum']              = pns_detail['data']['bossData'][1]['value']
    pns_result['transfiniteRefreshTimeStamp'] = calculate_time_stamp(pns_detail['data']['bossData'][1]['refreshTimeStamp'], current_timestamp)
    pns_result['arenaRefreshTimeStamp']       = calculate_time_stamp(pns_detail['data']['bossData'][2]['refreshTimeStamp'], current_timestamp)
    pns_result['arenaBlackCard']              = pns_detail['data']['bossData'][2]['value']
    pns_result['strongHoldRate']              = pns_detail['data']['bossData'][3]['value']
    pns_result['strongHoldTimeStamp']         = calculate_time_stamp(pns_detail['data']['bossData'][3]['refreshTimeStamp'], current_timestamp)
    data_pic = await pic_generator(pns_result)
    return(data_pic)


import httpx

header_data = {
        'osversion': 'Android',
        'devcode': 'A734EC22C2D3F93154BC2952A30ABF5A32F01753',
        'countryCode': 'CN',
        'pragma': 'no-cache',
        'cache-control': 'no-cache',
        'accept': 'application/json, text/plain, */*',
        'model': '2201122C',
        'source': 'android',
        'lang': 'zh-Hans',
        'version': '2.1.0',
        'versionCode': '2100',
        'token': '',
        'content-type': 'application/x-www-form-urlencoded; charset=utf-8',
        'x-requested-with': 'com.kurogame.kjq',
        'sec-fetch-site': 'same-site',
        'sec-fetch-mode': 'cors',
        'sec-fetch-dest': 'empty',
        'accept-encoding': 'gzip, deflate, br, zstd',
        'accept-language': 'zh-CN,zh;q=0.9,en-US;q=0.8,en;q=0.7',
        'user-agent': 'okhttp/3.10.0',
    }

def getPunishingResource(token):
    REFRESH_URL = 'https://api.kurobbs.com/gamer/widget/game2/refresh'
    header_data['token'] = token
    form_data = {
        'gameId': 2,
        'type': 2,
    }

    try:
        result = do_fetch(REFRESH_URL, header_data, form_data)
    except Exception as e:
        raise e
    return result

def get_punishing_account_info(token):
    PNS_INFO_URL = 'https://api.kurobbs.com/gamer/role/list'
    header_data['token'] = token
    form_data = {
        'gameId': 2,
    }

    try:
        result = do_fetch(PNS_INFO_URL, header_data, form_data)
    except Exception as e:
        raise e
    return result

def get_mc_account_info(token):

    MC_INFO_URL = 'https://api.kurobbs.com/gamer/role/list'
    header_data['token'] = token
    form_data = {
        'gameId': 3,
    }

    try:
        result = do_fetch(MC_INFO_URL, header_data, form_data)
    except Exception as e:
        raise e
    return result

def get_monthly_resource(roleId, token):
    MONTHLY_RESOURCE_URL = 'https://api.kurobbs.com/gamer/resource/month'
    header_data['token'] = token
    form_data = {
        'roleId': roleId,
    }
    try:
        result = do_fetch(MONTHLY_RESOURCE_URL, header_data, form_data)
    except Exception as e:
        raise e
    return result

def get_pns_game_account(roleId, serverId, token):
    GAME_ACCOUNT_URL = 'https://api.kurobbs.com/gamer/roleBox/gameAccount'
    header_data['token'] = token
    form_data = {
        'gameId' : 2,
        'roleId' : roleId,
        'serverId' : serverId,
    }
    try:
        result = do_fetch(GAME_ACCOUNT_URL, header_data, form_data)
    except Exception as e:
        raise e
    return result

def do_fetch(url, header, data):
    try:
        response = httpx.post(url, headers=header, data=data)
        if not (response.status_code == 200):
            raise Exception('fetch error: ', response.status_code, response.reason_phrase)
        else:
            rsp = response.json()

            if rsp.get('code') == 200:
                result = rsp
            else:
                raise Exception('api error:', rsp)
    except Exception as e:
        raise Exception('fetch error: '+ str(e))
    return result



import json
from ..pns_dao import UserInfoManagement
from ..token_judgement import token_judgement
from .pns_detail_handler import get_punishing_account_info, get_mc_account_info

manager = UserInfoManagement()
def pns_login_handler(user_id, data_content):

    if data_content == "":
        return("请输入战双token, 如：/战双登录 your_token")
    if token_judgement(data_content) != "success":
        return(token_judgement(data_content))

    pns_info  = get_punishing_account_info(json.loads(data_content)["data"]["token"])
    mc_info   = get_mc_account_info(json.loads(data_content)["data"]["token"])

    pns_id    = None if (not pns_info["data"])  else pns_info["data"][0]["roleId"]
    mc_id     = None if (not mc_info["data"])   else mc_info ["data"][0]["roleId"]

    if pns_id or mc_id:
        server_id = pns_info["data"][0]["serverId"]  if (pns_info["data"])  else mc_info["data"]["serverId"] if (mc_info["data"]) else None
        data = {
            'userId': user_id,
            'pnsId': pns_id,
            'mcId': mc_id,
            'token': str(data_content),
            'serverId': server_id
        }
        if manager._get_data(user_id):
            try:
                manager._update_data(user_id, data)
            except Exception as e:
                return("更新数据失败: " + str(e))
            return("token更新成功！")
        else:
            try:
                manager._insert_data(user_id, data)
            except Exception as e:
                return("插入数据失败: " + str(e))
        return("token保存成功！")
    return("该账号暂未绑定任何游戏")

def get_kuro_token(qq_id):
    result = manager._get_data(qq_id)
    return result